#!/usr/bin/env python
#
# USAGE: rho = m2003(s, t, p)
#
# Purpose
# -------
# Calculates the density of seawater, using the equation of state of:
#
# McDougall, T. J., D. R. Jackett, D. G. Wright and R. Feistel (2003), Accurate
# and Computationally Efficient Algorithms for Potential Temperature and
# Density of Seawater, Journal of Atmospheric and Oceanic Technology, 20,
# 730-741.
#
#
# Inputs
# ------
# s		Salinity (psu)
# t		Potential temperature (degC)
# p		Pressure (db)
#
# Outputs
# -------
# rho		Density (kg m-3)
#
# History
# -------
# 2009 May 8	Steven Phipps	Original version - Written in Fortran
#
# 2012 May 7    David Hutchinson  Adapted for use in matlab and converted to kg m-3
# 2014 September 9  David Hutchinson  Translated into python

import numpy as np

def m2003(s, t, p):
# List of coefficients for the rational function
    a0 = 9.99843699e2
    a1 = 7.35212840
    a2 = -5.45928211e-2
    a3 = 3.98476704e-4
    a4 = 2.96938239
    a5 = -7.23268813e-3
    a6 = 2.12382341e-3
    a7 = 1.04004591e-2
    a8 = 1.03970529e-7
    a9 = 5.18761880e-6
    a10 = -3.24041825e-8
    a11 = -1.23869360e-11
    b0 = 1.0
    b1 = 7.28606739e-3
    b2 = -4.60835542e-5
    b3 = 3.68390573e-7
    b4 = 1.80809186e-10
    b5 = 2.14691708e-3
    b6 = -9.27062484e-6
    b7 = -1.78343643e-10
    b8 = 4.76534122e-6
    b9 = 1.63410736e-9
    b10 = 5.30848875e-6
    b11 = -3.03175128e-16
    b12 = -1.27934137e-17

    # Pressure terms
    p2 = p * p
    p3 = p * p2

    # Salinity terms
    s15 = np.ma.power(s,1.5)
    s2 = s * s

    # Temperature terms
    t2 = t * t
    t3 = t2 * t
    t4 = t3 * t

    # Calculate Density
    rho = (a0 + a1*t + a2*t2 + a3*t3 + a4*s + 
           a5*s*t + a6*s2 + a7*p + a8*p*t2 + 
           a9*p*s + a10*p2 + a11*p2*t2) / \
          (b0 + b1*t + b2*t2 + b3*t3 + b4*t4 + 
           b5*s + b6*s*t + b7*s*t3 + b8*s15 + 
           b9*s15*t2 + b10*p + b11*p2*t3 + b12*p3*t)

    return rho
